@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2020, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -32,18 +32,21 @@ pub trait Muxer: Send {
   fn flush(&mut self) -> io::Result<()>;
 }
 
-pub fn create_muxer(
-  path: &str, overwrite: bool,
+pub fn create_muxer<P: AsRef<Path>>(
+  path: P, overwrite: bool,
 ) -> Result<Box<dyn Muxer + Send>, CliError> {
   if !overwrite {
-    IvfMuxer::check_file(path)?;
+    IvfMuxer::check_file(path.as_ref())?;
   }
 
-  if path == "-" {
-    return IvfMuxer::open(path);
+  if let Some(path) = path.as_ref().to_str() {
+    if path == "-" {
+      return IvfMuxer::open(path);
+    }
   }
 
-  let ext = Path::new(path)
+  let ext = path
+    .as_ref()
     .extension()
     .and_then(OsStr::to_str)
     .map(str::to_lowercase)
@@ -52,7 +55,7 @@ pub fn create_muxer(
   match &ext[..] {
     "ivf" => IvfMuxer::open(path),
     _e => {
-      panic!("{} is not a supported extension, please change to .ivf", ext);
+      panic!("{ext} is not a supported extension, please change to .ivf");
     }
   }
 }
