@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2023, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -7,36 +7,79 @@
 // Media Patent License 1.0 was not distributed with this source code in the
 // PATENTS file, you can obtain it at www.aomedia.org/license/patent.
 
+// Safety lints
 #![deny(bare_trait_objects)]
-#![allow(clippy::cast_lossless)]
-#![allow(clippy::cast_ptr_alignment)]
-#![allow(clippy::cognitive_complexity)]
-#![allow(clippy::needless_range_loop)]
-#![allow(clippy::too_many_arguments)]
-#![allow(clippy::verbose_bit_mask)]
-#![allow(clippy::unreadable_literal)]
-#![allow(clippy::many_single_char_names)]
+#![deny(clippy::as_ptr_cast_mut)]
+#![deny(clippy::large_stack_arrays)]
 // Performance lints
+#![warn(clippy::inefficient_to_string)]
+#![warn(clippy::invalid_upcast_comparisons)]
+#![warn(clippy::iter_with_drain)]
 #![warn(clippy::linkedlist)]
-#![warn(clippy::missing_const_for_fn)]
 #![warn(clippy::mutex_integer)]
+#![warn(clippy::naive_bytecount)]
+#![warn(clippy::needless_bitwise_bool)]
+#![warn(clippy::needless_collect)]
+#![warn(clippy::or_fun_call)]
+#![warn(clippy::stable_sort_primitive)]
 #![warn(clippy::suboptimal_flops)]
+#![warn(clippy::trivial_regex)]
+#![warn(clippy::trivially_copy_pass_by_ref)]
+#![warn(clippy::unnecessary_join)]
+#![warn(clippy::unused_async)]
+#![warn(clippy::zero_sized_map_values)]
 // Correctness lints
-#![warn(clippy::expl_impl_clone_on_copy)]
-#![warn(clippy::mem_forget)]
-#![warn(clippy::path_buf_push_overwrite)]
+#![deny(clippy::case_sensitive_file_extension_comparisons)]
+#![deny(clippy::copy_iterator)]
+#![deny(clippy::expl_impl_clone_on_copy)]
+#![deny(clippy::float_cmp)]
+#![warn(clippy::imprecise_flops)]
+#![deny(clippy::manual_instant_elapsed)]
+#![deny(clippy::mem_forget)]
+#![deny(clippy::path_buf_push_overwrite)]
+#![deny(clippy::same_functions_in_if_condition)]
+#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unicode_not_nfc)]
 // Clarity/formatting lints
+#![warn(clippy::checked_conversions)]
+#![allow(clippy::comparison_chain)]
+#![warn(clippy::derive_partial_eq_without_eq)]
+#![allow(clippy::enum_variant_names)]
+#![warn(clippy::explicit_deref_methods)]
+#![warn(clippy::filter_map_next)]
+#![warn(clippy::flat_map_option)]
+#![warn(clippy::fn_params_excessive_bools)]
+#![warn(clippy::implicit_clone)]
+#![warn(clippy::iter_not_returning_iterator)]
+#![warn(clippy::iter_on_empty_collections)]
+#![warn(clippy::macro_use_imports)]
+#![warn(clippy::manual_clamp)]
+#![warn(clippy::manual_let_else)]
+#![warn(clippy::manual_ok_or)]
+#![warn(clippy::manual_string_new)]
 #![warn(clippy::map_flatten)]
+#![warn(clippy::match_bool)]
 #![warn(clippy::mut_mut)]
 #![warn(clippy::needless_borrow)]
 #![warn(clippy::needless_continue)]
+#![allow(clippy::needless_range_loop)]
+#![allow(clippy::too_many_arguments)]
+#![warn(clippy::range_minus_one)]
 #![warn(clippy::range_plus_one)]
+#![warn(clippy::ref_binding_to_reference)]
+#![warn(clippy::ref_option_ref)]
+#![warn(clippy::trait_duplication_in_bounds)]
+#![warn(clippy::unused_peekable)]
+#![warn(clippy::unused_rounding)]
+#![warn(clippy::unused_self)]
+#![allow(clippy::upper_case_acronyms)]
+#![warn(clippy::verbose_bit_mask)]
+#![warn(clippy::verbose_file_reads)]
 // Documentation lints
+#![warn(clippy::doc_link_with_quotes)]
 #![warn(clippy::doc_markdown)]
 #![warn(clippy::missing_errors_doc)]
 #![warn(clippy::missing_panics_doc)]
-// FIXME: Temporarily disabled due to https://github.com/rust-lang/rust-clippy/issues/9142
-#![allow(clippy::undocumented_unsafe_blocks)]
 
 #[macro_use]
 extern crate log;
@@ -314,17 +357,20 @@ fn do_encode<T: Pixel, D: Decoder>(
 }
 
 fn main() -> Result<(), Box<dyn std::error::Error>> {
-  #[cfg(feature = "tracing")]
-  use rust_hawktracer::*;
   init_logger();
 
   #[cfg(feature = "tracing")]
-  let instance = HawktracerInstance::new();
+  let (chrome_layer, _guard) =
+    tracing_chrome::ChromeLayerBuilder::new().build();
+
   #[cfg(feature = "tracing")]
-  let _listener = instance.create_listener(HawktracerListenerType::ToFile {
-    file_path: "trace.bin".into(),
-    buffer_size: 4096,
-  });
+  {
+    use tracing_subscriber::layer::subscriberext;
+    tracing::subscriber::set_global_default(
+      tracing_subscriber::registry().with(chrome_layer),
+    )
+    .unwrap();
+  }
 
   run().map_err(|e| {
     error::print_error(&e);
@@ -377,7 +423,7 @@ fn init_logger() {
 cfg_if::cfg_if! {
   if #[cfg(any(target_os = "windows", target_arch = "wasm32"))] {
     fn print_rusage() {
-      eprintln!("Windows benchmarking is not supported currently.");
+      eprintln!("Resource usage reporting is not currently supported on this platform");
     }
   } else {
     fn print_rusage() {
@@ -595,7 +641,7 @@ fn run() -> Result<(), error::CliError> {
 
   let source = Source::new(cli.limit, y4m_dec);
 
-  if video_info.bit_depth == 8 {
+  if video_info.bit_depth == 8 && !cli.force_highbitdepth {
     do_encode::<u8, y4m::Decoder<Box<dyn Read + Send>>>(
       cfg,
       cli.verbose,
