@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2018-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -17,23 +17,13 @@ use crate::quantize::*;
 use crate::tiling::*;
 use crate::util::{clamp, ILog, Pixel};
 use crate::DeblockState;
-use rust_hawktracer::*;
+use rayon::iter::*;
 use std::cmp;
 
-use crate::rayon::iter::*;
-
 fn deblock_adjusted_level(
   deblock: &DeblockState, block: &Block, pli: usize, vertical: bool,
 ) -> usize {
-  let idx = if pli == 0 {
-    if vertical {
-      0
-    } else {
-      1
-    }
-  } else {
-    pli + 1
-  };
+  let idx = if pli == 0 { usize::from(!vertical) } else { pli + 1 };
 
   let level = if deblock.block_deltas_enabled {
     // By-block filter strength delta, if the feature is active.
@@ -58,12 +48,9 @@ fn deblock_adjusted_level(
   if deblock.deltas_enabled {
     let mode = block.mode;
     let reference = block.ref_frames[0];
-    let mode_type =
-      if mode >= NEARESTMV && mode != GLOBALMV && mode != GLOBAL_GLOBALMV {
-        1
-      } else {
-        0
-      };
+    let mode_type = usize::from(
+      mode >= NEARESTMV && mode != GLOBALMV && mode != GLOBAL_GLOBALMV,
+    );
     let l5 = level >> 5;
     clamp(
       level as i32
@@ -339,12 +326,8 @@ fn copy_vertical<T: Pixel>(
 }
 
 #[inline]
-fn stride_sse(a: &[i32], b: &[i32]) -> i64 {
-  let mut acc: i32 = 0;
-  for (a, b) in a.iter().take(b.len()).zip(b) {
-    acc += (*a - *b) * (*a - *b)
-  }
-  acc as i64
+fn stride_sse<const LEN: usize>(a: &[i32; LEN], b: &[i32; LEN]) -> i64 {
+  a.iter().zip(b).map(|(a, b)| (a - b) * (a - b)).sum::<i32>() as i64
 }
 
 #[inline]
@@ -465,10 +448,8 @@ fn sse_size4<T: Pixel>(
 
     // mask4 sets the dividing line for filter vs no filter
     // nhev4 sets the dividing line between narrow2 and narrow4
-    let mask =
-      clamp(mask4(p1, p0, q0, q1, bd - 8), 1, MAX_LOOP_FILTER + 1) as usize;
-    let nhev =
-      clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1) as usize;
+    let mask = clamp(mask4(p1, p0, q0, q1, bd - 8), 1, MAX_LOOP_FILTER + 1);
+    let nhev = clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1);
 
     // sse for each; short-circuit the 'special' no-op cases.
     let sse_none = stride_sse(&a, &none);
@@ -611,11 +592,9 @@ fn sse_size6<T: Pixel>(
     // flat6 decides between wide and narrow filters (unrelated to level)
     // nhev4 sets the dividing line between narrow2 and narrow4
     let mask =
-      clamp(mask6(p2, p1, p0, q0, q1, q2, bd - 8), 1, MAX_LOOP_FILTER + 1)
-        as usize;
+      clamp(mask6(p2, p1, p0, q0, q1, q2, bd - 8), 1, MAX_LOOP_FILTER + 1);
     let flatp = flat6(p2, p1, p0, q0, q1, q2) <= flat;
-    let nhev =
-      clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1) as usize;
+    let nhev = clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1);
 
     // sse for each; short-circuit the 'special' no-op cases.
     let sse_none = stride_sse(&a, &none);
@@ -815,10 +794,9 @@ fn sse_size8<T: Pixel>(
       mask8(p3, p2, p1, p0, q0, q1, q2, q3, bd - 8),
       1,
       MAX_LOOP_FILTER + 1,
-    ) as usize;
+    );
     let flatp = flat8(p3, p2, p1, p0, q0, q1, q2, q3) <= flat;
-    let nhev =
-      clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1) as usize;
+    let nhev = clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1);
 
     // sse for each; short-circuit the 'special' no-op cases.
     let sse_none = stride_sse(&a, &none);
@@ -1073,11 +1051,10 @@ fn sse_size14<T: Pixel>(
       mask8(p3, p2, p1, p0, q0, q1, q2, q3, bd - 8),
       1,
       MAX_LOOP_FILTER + 1,
-    ) as usize;
+    );
     let flat8p = flat8(p3, p2, p1, p0, q0, q1, q2, q3) <= flat;
     let flat14p = flat14_outer(p6, p5, p4, p0, q0, q4, q5, q6) <= flat;
-    let nhev =
-      clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1) as usize;
+    let nhev = clamp(nhev4(p1, p0, q0, q1, bd - 8), mask, MAX_LOOP_FILTER + 1);
 
     // sse for each; short-circuit the 'special' no-op cases.
     let sse_none = stride_sse(&a, &none);
@@ -1313,7 +1290,7 @@ fn sse_h_edge<T: Pixel>(
 }
 
 // Deblocks all edges, vertical and horizontal, in a single plane
-#[hawktracer(deblock_plane)]
+#[profiling::function]
 pub fn deblock_plane<T: Pixel>(
   deblock: &DeblockState, p: &mut PlaneRegionMut<T>, pli: usize,
   blocks: &TileBlocks, crop_w: usize, crop_h: usize, bd: usize,
@@ -1563,16 +1540,14 @@ fn sse_plane<T: Pixel>(
 }
 
 // Deblocks all edges in all planes of a frame
-#[hawktracer(deblock_filter_frame)]
+#[profiling::function]
 pub fn deblock_filter_frame<T: Pixel>(
   deblock: &DeblockState, tile: &mut TileMut<T>, blocks: &TileBlocks,
   crop_w: usize, crop_h: usize, bd: usize, planes: usize,
 ) {
-  (&mut tile.planes[..planes]).par_iter_mut().enumerate().for_each(
-    |(pli, plane)| {
-      deblock_plane(deblock, plane, pli, blocks, crop_w, crop_h, bd);
-    },
-  );
+  tile.planes[..planes].par_iter_mut().enumerate().for_each(|(pli, plane)| {
+    deblock_plane(deblock, plane, pli, blocks, crop_w, crop_h, bd);
+  });
 }
 
 fn sse_optimize<T: Pixel>(
@@ -1581,8 +1556,8 @@ fn sse_optimize<T: Pixel>(
 ) -> [u8; 4] {
   // i64 allows us to accumulate a total of ~ 35 bits worth of pixels
   assert!(
-    input.planes[0].plane_cfg.width.ilog()
-      + input.planes[0].plane_cfg.height.ilog()
+    ILog::ilog(input.planes[0].plane_cfg.width)
+      + ILog::ilog(input.planes[0].plane_cfg.height)
       < 35
   );
   let mut level = [0; 4];
@@ -1641,13 +1616,13 @@ fn sse_optimize<T: Pixel>(
   level
 }
 
-#[hawktracer(deblock_filter_optimize)]
+#[profiling::function]
 pub fn deblock_filter_optimize<T: Pixel, U: Pixel>(
   fi: &FrameInvariants<T>, rec: &Tile<U>, input: &Tile<U>,
   blocks: &TileBlocks, crop_w: usize, crop_h: usize,
 ) -> [u8; 4] {
   if fi.config.speed_settings.fast_deblock {
-    let q = ac_q(fi.base_q_idx, 0, fi.sequence.bit_depth) as i32;
+    let q = ac_q(fi.base_q_idx, 0, fi.sequence.bit_depth).get() as i32;
     let level = clamp(
       match fi.sequence.bit_depth {
         8 => {
