@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -15,6 +15,7 @@ cfg_if::cfg_if! {
   }
 }
 
+use crate::api::SGRComplexityLevel;
 use crate::color::ChromaSampling::Cs400;
 use crate::context::{MAX_PLANES, SB_SIZE};
 use crate::encoder::FrameInvariants;
@@ -23,9 +24,6 @@ use crate::frame::{
 };
 use crate::tiling::{Area, PlaneRegion, PlaneRegionMut, Rect};
 use crate::util::{clamp, CastFromPrimitive, ILog, Pixel};
-use rust_hawktracer::*;
-
-use crate::api::SGRComplexityLevel;
 use std::cmp;
 use std::iter::FusedIterator;
 use std::ops::{Index, IndexMut};
@@ -86,7 +84,7 @@ const SGRPROJ_ALL_SETS: &[u8] =
 // 1st, 3rd, ... smallest params of each group.
 const SGRPROJ_REDUCED_SETS: &[u8] = &[1, 3, 5, 7, 9, 11, 13, 15];
 
-pub fn get_sgr_sets(complexity: SGRComplexityLevel) -> &'static [u8] {
+pub const fn get_sgr_sets(complexity: SGRComplexityLevel) -> &'static [u8] {
   match complexity {
     SGRComplexityLevel::Full => SGRPROJ_ALL_SETS,
     SGRComplexityLevel::Reduced => SGRPROJ_REDUCED_SETS,
@@ -123,21 +121,21 @@ impl IntegralImageBuffer {
 }
 
 #[allow(unused)] // Wiener coming soon!
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Default)]
 pub enum RestorationFilter {
+  #[default]
   None,
-  Wiener { coeffs: [[i8; 3]; 2] },
-  Sgrproj { set: u8, xqd: [i8; 2] },
-}
-
-impl Default for RestorationFilter {
-  fn default() -> RestorationFilter {
-    RestorationFilter::None
-  }
+  Wiener {
+    coeffs: [[i8; 3]; 2],
+  },
+  Sgrproj {
+    set: u8,
+    xqd: [i8; 2],
+  },
 }
 
 impl RestorationFilter {
-  pub fn notequal(self, cmp: RestorationFilter) -> bool {
+  pub const fn notequal(self, cmp: RestorationFilter) -> bool {
     match self {
       RestorationFilter::None {} => !matches!(cmp, RestorationFilter::None {}),
       RestorationFilter::Sgrproj { set, xqd } => {
@@ -174,32 +172,39 @@ pub(crate) mod rust {
   use crate::Pixel;
 
   #[inline(always)]
-  pub(crate) fn sgrproj_box_ab_internal(
+  pub(crate) fn sgrproj_box_ab_internal<const BD: usize>(
     r: usize, af: &mut [u32], bf: &mut [u32], iimg: &[u32], iimg_sq: &[u32],
     iimg_stride: usize, start_x: usize, y: usize, stripe_w: usize, s: u32,
-    bdm8: usize,
   ) {
     let d: usize = r * 2 + 1;
     let n: usize = d * d;
     let one_over_n = if r == 1 { 455 } else { 164 };
 
+    assert!(iimg.len() > (y + d) * iimg_stride + stripe_w + 1 + d);
+    assert!(iimg_sq.len() > (y + d) * iimg_stride + stripe_w + 1 + d);
+    assert!(af.len() > stripe_w + 1);
+    assert!(bf.len() > stripe_w + 1);
+
     for x in start_x..stripe_w + 2 {
-      let sum = get_integral_square(iimg, iimg_stride, x, y, d);
-      let ssq = get_integral_square(iimg_sq, iimg_stride, x, y, d);
-      let (reta, retb) =
-        sgrproj_sum_finish(ssq, sum, n as u32, one_over_n, s, bdm8);
-      af[x] = reta;
-      bf[x] = retb;
+      // SAFETY: We perform the bounds checks above, once for the whole loop
+      unsafe {
+        let sum = get_integral_square(iimg, iimg_stride, x, y, d);
+        let ssq = get_integral_square(iimg_sq, iimg_stride, x, y, d);
+        let (reta, retb) =
+          sgrproj_sum_finish::<BD>(ssq, sum, n as u32, one_over_n, s);
+        *af.get_unchecked_mut(x) = reta;
+        *bf.get_unchecked_mut(x) = retb;
+      }
     }
   }
 
   // computes an intermediate (ab) row for stripe_w + 2 columns at row y
-  pub(crate) fn sgrproj_box_ab_r1(
+  pub(crate) fn sgrproj_box_ab_r1<const BD: usize>(
     af: &mut [u32], bf: &mut [u32], iimg: &[u32], iimg_sq: &[u32],
-    iimg_stride: usize, y: usize, stripe_w: usize, s: u32, bdm8: usize,
+    iimg_stride: usize, y: usize, stripe_w: usize, s: u32,
     _cpu: CpuFeatureLevel,
   ) {
-    sgrproj_box_ab_internal(
+    sgrproj_box_ab_internal::<BD>(
       1,
       af,
       bf,
@@ -210,17 +215,16 @@ pub(crate) mod rust {
       y,
       stripe_w,
       s,
-      bdm8,
     );
   }
 
   // computes an intermediate (ab) row for stripe_w + 2 columns at row y
-  pub(crate) fn sgrproj_box_ab_r2(
+  pub(crate) fn sgrproj_box_ab_r2<const BD: usize>(
     af: &mut [u32], bf: &mut [u32], iimg: &[u32], iimg_sq: &[u32],
-    iimg_stride: usize, y: usize, stripe_w: usize, s: u32, bdm8: usize,
+    iimg_stride: usize, y: usize, stripe_w: usize, s: u32,
     _cpu: CpuFeatureLevel,
   ) {
-    sgrproj_box_ab_internal(
+    sgrproj_box_ab_internal::<BD>(
       2,
       af,
       bf,
@@ -231,7 +235,6 @@ pub(crate) mod rust {
       y,
       stripe_w,
       s,
-      bdm8,
     );
   }
 
@@ -338,14 +341,13 @@ pub(crate) mod rust {
 }
 
 #[inline(always)]
-fn sgrproj_sum_finish(
-  ssq: u32, sum: u32, n: u32, one_over_n: u32, s: u32, bdm8: usize,
+fn sgrproj_sum_finish<const BD: usize>(
+  ssq: u32, sum: u32, n: u32, one_over_n: u32, s: u32,
 ) -> (u32, u32) {
+  let bdm8 = BD - 8;
   let scaled_ssq = (ssq + (1 << (2 * bdm8) >> 1)) >> (2 * bdm8);
   let scaled_sum = (sum + (1 << bdm8 >> 1)) >> bdm8;
-  let p =
-    cmp::max(0, (scaled_ssq * n) as i32 - (scaled_sum * scaled_sum) as i32)
-      as u32;
+  let p = (scaled_ssq * n).saturating_sub(scaled_sum * scaled_sum);
   let z = (p * s + (1 << SGRPROJ_MTABLE_BITS >> 1)) >> SGRPROJ_MTABLE_BITS;
   let a = if z >= 255 {
     256
@@ -359,14 +361,20 @@ fn sgrproj_sum_finish(
 }
 
 // Using an integral image, compute the sum of a square region
-fn get_integral_square(
+// SAFETY: The size of `iimg` must be at least `(y + size) * stride + x + size`
+#[inline(always)]
+unsafe fn get_integral_square(
   iimg: &[u32], stride: usize, x: usize, y: usize, size: usize,
 ) -> u32 {
   // Cancel out overflow in iimg by using wrapping arithmetic
-  iimg[y * stride + x]
-    .wrapping_add(iimg[(y + size) * stride + x + size])
-    .wrapping_sub(iimg[(y + size) * stride + x])
-    .wrapping_sub(iimg[y * stride + x + size])
+  let top_left = *iimg.get_unchecked(y * stride + x);
+  let top_right = *iimg.get_unchecked(y * stride + x + size);
+  let bottom_left = *iimg.get_unchecked((y + size) * stride + x);
+  let bottom_right = *iimg.get_unchecked((y + size) * stride + x + size);
+  top_left
+    .wrapping_add(bottom_right)
+    .wrapping_sub(bottom_left)
+    .wrapping_sub(top_right)
 }
 
 struct VertPaddedIter<'a, T: Pixel> {
@@ -390,7 +398,7 @@ struct VertPaddedIter<'a, T: Pixel> {
   crop: isize,
 }
 
-impl<'a, 'b, T: Pixel> VertPaddedIter<'a, T> {
+impl<'a, T: Pixel> VertPaddedIter<'a, T> {
   fn new(
     cdeffed: &PlaneSlice<'a, T>, deblocked: &PlaneSlice<'a, T>,
     stripe_h: usize, crop: usize,
@@ -443,12 +451,11 @@ impl<'a, T: Pixel> Iterator for VertPaddedIter<'a, T> {
       let ly = clamp(cropped_y, self.stripe_begin - 2, self.stripe_end + 1);
 
       // decide if we're vertically inside or outside the strip
-      let src_plane =
-        if ly >= self.stripe_begin && ly < self.stripe_end as isize {
-          self.cdeffed
-        } else {
-          self.deblocked
-        };
+      let src_plane = if ly >= self.stripe_begin && ly < self.stripe_end {
+        self.cdeffed
+      } else {
+        self.deblocked
+      };
       // cannot directly return self.ps.row(row) due to lifetime issue
       let range = src_plane.row_range(self.x, ly);
       self.y += 1;
@@ -518,6 +525,7 @@ impl<'a, T: Pixel> Iterator for HorzPaddedIter<'a, T> {
 impl<T: Pixel> ExactSizeIterator for HorzPaddedIter<'_, T> {}
 impl<T: Pixel> FusedIterator for HorzPaddedIter<'_, T> {}
 
+#[profiling::function]
 pub fn setup_integral_image<T: Pixel>(
   integral_image_buffer: &mut IntegralImageBuffer,
   integral_image_stride: usize, crop_w: usize, crop_h: usize, stripe_w: usize,
@@ -617,13 +625,13 @@ pub fn setup_integral_image<T: Pixel>(
   }
 }
 
+#[profiling::function]
 pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
   set: u8, xqd: [i8; 2], fi: &FrameInvariants<T>,
   integral_image_buffer: &IntegralImageBuffer, integral_image_stride: usize,
   cdeffed: &PlaneSlice<U>, out: &mut PlaneRegionMut<U>,
 ) {
   let &Rect { width: stripe_w, height: stripe_h, .. } = out.rect();
-  let bdm8 = fi.sequence.bit_depth - 8;
   let mut a_r2: [[u32; IMAGE_WIDTH_MAX + 2]; 2] =
     [[0; IMAGE_WIDTH_MAX + 2]; 2];
   let mut b_r2: [[u32; IMAGE_WIDTH_MAX + 2]; 2] =
@@ -639,6 +647,19 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
   let s_r2: u32 = SGRPROJ_PARAMS_S[set as usize][0];
   let s_r1: u32 = SGRPROJ_PARAMS_S[set as usize][1];
 
+  let fn_ab_r1 = match fi.sequence.bit_depth {
+    8 => sgrproj_box_ab_r1::<8>,
+    10 => sgrproj_box_ab_r1::<10>,
+    12 => sgrproj_box_ab_r1::<12>,
+    _ => unimplemented!(),
+  };
+  let fn_ab_r2 = match fi.sequence.bit_depth {
+    8 => sgrproj_box_ab_r2::<8>,
+    10 => sgrproj_box_ab_r2::<10>,
+    12 => sgrproj_box_ab_r2::<12>,
+    _ => unimplemented!(),
+  };
+
   /* prime the intermediate arrays */
   // One oddness about the radius=2 intermediate array computations that
   // the spec doesn't make clear: Although the spec defines computation
@@ -647,7 +668,7 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
   let integral_image = &integral_image_buffer.integral_image;
   let sq_integral_image = &integral_image_buffer.sq_integral_image;
   if s_r2 > 0 {
-    sgrproj_box_ab_r2(
+    fn_ab_r2(
       &mut a_r2[0],
       &mut b_r2[0],
       integral_image,
@@ -656,13 +677,12 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
       0,
       stripe_w,
       s_r2,
-      bdm8,
       fi.cpu_feature_level,
     );
   }
   if s_r1 > 0 {
     let integral_image_offset = integral_image_stride + 1;
-    sgrproj_box_ab_r1(
+    fn_ab_r1(
       &mut a_r1[0],
       &mut b_r1[0],
       &integral_image[integral_image_offset..],
@@ -671,10 +691,9 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
       0,
       stripe_w,
       s_r1,
-      bdm8,
       fi.cpu_feature_level,
     );
-    sgrproj_box_ab_r1(
+    fn_ab_r1(
       &mut a_r1[1],
       &mut b_r1[1],
       &integral_image[integral_image_offset..],
@@ -683,7 +702,6 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
       1,
       stripe_w,
       s_r1,
-      bdm8,
       fi.cpu_feature_level,
     );
   }
@@ -694,7 +712,7 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
   for y in (0..stripe_h).step_by(2) {
     // get results to use y and y+1
     let f_r2_ab: [&[u32]; 2] = if s_r2 > 0 {
-      sgrproj_box_ab_r2(
+      fn_ab_r2(
         &mut a_r2[(y / 2 + 1) % 2],
         &mut b_r2[(y / 2 + 1) % 2],
         integral_image,
@@ -703,7 +721,6 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
         y + 2,
         stripe_w,
         s_r2,
-        bdm8,
         fi.cpu_feature_level,
       );
       let ap0: [&[u32]; 2] = [&a_r2[(y / 2) % 2], &a_r2[(y / 2 + 1) % 2]];
@@ -734,7 +751,7 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
       let y = y + dy;
       if s_r1 > 0 {
         let integral_image_offset = integral_image_stride + 1;
-        sgrproj_box_ab_r1(
+        fn_ab_r1(
           &mut a_r1[(y + 2) % 3],
           &mut b_r1[(y + 2) % 3],
           &integral_image[integral_image_offset..],
@@ -743,7 +760,6 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
           y + 2,
           stripe_w,
           s_r1,
-          bdm8,
           fi.cpu_feature_level,
         );
         let ap1: [&[u32]; 3] =
@@ -826,13 +842,12 @@ pub fn sgrproj_stripe_filter<T: Pixel, U: Pixel>(
 
 // Input params follow the same rules as sgrproj_stripe_filter.
 // Inputs are relative to the colocated slice views.
+#[profiling::function]
 pub fn sgrproj_solve<T: Pixel>(
   set: u8, fi: &FrameInvariants<T>,
   integral_image_buffer: &IntegralImageBuffer, input: &PlaneRegion<'_, T>,
   cdeffed: &PlaneSlice<T>, cdef_w: usize, cdef_h: usize,
 ) -> (i8, i8) {
-  let bdm8 = fi.sequence.bit_depth - 8;
-
   let mut a_r2: [[u32; IMAGE_WIDTH_MAX + 2]; 2] =
     [[0; IMAGE_WIDTH_MAX + 2]; 2];
   let mut b_r2: [[u32; IMAGE_WIDTH_MAX + 2]; 2] =
@@ -851,6 +866,19 @@ pub fn sgrproj_solve<T: Pixel>(
   let mut h: [[f64; 2]; 2] = [[0., 0.], [0., 0.]];
   let mut c: [f64; 2] = [0., 0.];
 
+  let fn_ab_r1 = match fi.sequence.bit_depth {
+    8 => sgrproj_box_ab_r1::<8>,
+    10 => sgrproj_box_ab_r1::<10>,
+    12 => sgrproj_box_ab_r1::<12>,
+    _ => unimplemented!(),
+  };
+  let fn_ab_r2 = match fi.sequence.bit_depth {
+    8 => sgrproj_box_ab_r2::<8>,
+    10 => sgrproj_box_ab_r2::<10>,
+    12 => sgrproj_box_ab_r2::<12>,
+    _ => unimplemented!(),
+  };
+
   /* prime the intermediate arrays */
   // One oddness about the radius=2 intermediate array computations that
   // the spec doesn't make clear: Although the spec defines computation
@@ -859,7 +887,7 @@ pub fn sgrproj_solve<T: Pixel>(
   let integral_image = &integral_image_buffer.integral_image;
   let sq_integral_image = &integral_image_buffer.sq_integral_image;
   if s_r2 > 0 {
-    sgrproj_box_ab_r2(
+    fn_ab_r2(
       &mut a_r2[0],
       &mut b_r2[0],
       integral_image,
@@ -868,13 +896,12 @@ pub fn sgrproj_solve<T: Pixel>(
       0,
       cdef_w,
       s_r2,
-      bdm8,
       fi.cpu_feature_level,
     );
   }
   if s_r1 > 0 {
     let integral_image_offset = SOLVE_IMAGE_STRIDE + 1;
-    sgrproj_box_ab_r1(
+    fn_ab_r1(
       &mut a_r1[0],
       &mut b_r1[0],
       &integral_image[integral_image_offset..],
@@ -883,10 +910,9 @@ pub fn sgrproj_solve<T: Pixel>(
       0,
       cdef_w,
       s_r1,
-      bdm8,
       fi.cpu_feature_level,
     );
-    sgrproj_box_ab_r1(
+    fn_ab_r1(
       &mut a_r1[1],
       &mut b_r1[1],
       &integral_image[integral_image_offset..],
@@ -895,7 +921,6 @@ pub fn sgrproj_solve<T: Pixel>(
       1,
       cdef_w,
       s_r1,
-      bdm8,
       fi.cpu_feature_level,
     );
   }
@@ -906,7 +931,7 @@ pub fn sgrproj_solve<T: Pixel>(
   for y in (0..cdef_h).step_by(2) {
     // get results to use y and y+1
     let f_r2_01: [&[u32]; 2] = if s_r2 > 0 {
-      sgrproj_box_ab_r2(
+      fn_ab_r2(
         &mut a_r2[(y / 2 + 1) % 2],
         &mut b_r2[(y / 2 + 1) % 2],
         integral_image,
@@ -915,7 +940,6 @@ pub fn sgrproj_solve<T: Pixel>(
         y + 2,
         cdef_w,
         s_r2,
-        bdm8,
         fi.cpu_feature_level,
       );
       let ap0: [&[u32]; 2] = [&a_r2[(y / 2) % 2], &a_r2[(y / 2 + 1) % 2]];
@@ -940,7 +964,7 @@ pub fn sgrproj_solve<T: Pixel>(
       let y = y + dy;
       if s_r1 > 0 {
         let integral_image_offset = SOLVE_IMAGE_STRIDE + 1;
-        sgrproj_box_ab_r1(
+        fn_ab_r1(
           &mut a_r1[(y + 2) % 3],
           &mut b_r1[(y + 2) % 3],
           &integral_image[integral_image_offset..],
@@ -949,7 +973,6 @@ pub fn sgrproj_solve<T: Pixel>(
           y + 2,
           cdef_w,
           s_r1,
-          bdm8,
           fi.cpu_feature_level,
         );
         let ap1: [&[u32]; 3] =
@@ -1049,13 +1072,13 @@ pub fn sgrproj_solve<T: Pixel>(
       ((c[0] / h[0][0]).round() as i32, 0)
     }
   } else {
-    let det = h[0][0] * h[1][1] - h[0][1] * h[1][0];
+    let det = h[0][0].mul_add(h[1][1], -h[0][1] * h[1][0]);
     if det == 0. {
       (0, 0)
     } else {
       // If scaling up dividend would overflow, instead scale down the divisor
-      let div1 = h[1][1] * c[0] - h[0][1] * c[1];
-      let div2 = h[0][0] * c[1] - h[1][0] * c[0];
+      let div1 = h[1][1].mul_add(c[0], -h[0][1] * c[1]);
+      let div2 = h[0][0].mul_add(c[1], -h[1][0] * c[0]);
       ((div1 / det).round() as i32, (div2 / det).round() as i32)
     }
   };
@@ -1071,6 +1094,7 @@ pub fn sgrproj_solve<T: Pixel>(
   }
 }
 
+#[profiling::function]
 fn wiener_stripe_filter<T: Pixel>(
   coeffs: [[i8; 3]; 2], fi: &FrameInvariants<T>, crop_w: usize, crop_h: usize,
   stripe_w: usize, stripe_h: usize, stripe_x: usize, stripe_y: isize,
@@ -1151,7 +1175,7 @@ fn wiener_stripe_filter<T: Pixel>(
       let off = 3 - (xi as isize);
       let s = cmp::max(0, off) as usize;
       let s1 = (s as isize - off) as usize;
-      let n1 = (n as isize - off) as usize;
+      let n1 = (n - off) as usize;
 
       for (hf, &v) in hfilter[s..n as usize].iter().zip(src[s1..n1].iter()) {
         acc += hf * i32::cast_from(v);
@@ -1302,7 +1326,7 @@ impl RestorationState {
   pub fn new<T: Pixel>(fi: &FrameInvariants<T>, input: &Frame<T>) -> Self {
     let PlaneConfig { xdec, ydec, .. } = input.planes[1].cfg;
     // stripe size is decimated in 4:2:0 (and only 4:2:0)
-    let stripe_uv_decimate = if xdec > 0 && ydec > 0 { 1 } else { 0 };
+    let stripe_uv_decimate = usize::from(xdec > 0 && ydec > 0);
     let y_sb_log2 = if fi.sequence.use_128x128_superblock { 7 } else { 6 };
     let uv_sb_h_log2 = y_sb_log2 - xdec;
     let uv_sb_v_log2 = y_sb_log2 - ydec;
@@ -1341,11 +1365,9 @@ impl RestorationState {
             <= lrf_unit_size / 4
             || ((fi.height >> ydec) - 1) % (lrf_unit_size >> 1)
               <= lrf_unit_size / 4;
-          if unshifted_stretch && !shifted_stretch {
-            1 // shift to eliminate stretch
-          } else {
-            0 // don't shift; save the signaling bits
-          }
+          // shift to eliminate stretch if needed,
+          // otherwise do not shift and save the signaling bits
+          usize::from(unshifted_stretch && !shifted_stretch)
         }
       } else {
         0
@@ -1461,7 +1483,7 @@ impl RestorationState {
     }
   }
 
-  #[hawktracer(lrf_filter_frame)]
+  #[profiling::function]
   pub fn lrf_filter_frame<T: Pixel>(
     &mut self, out: &mut Frame<T>, pre_cdef: &Frame<T>,
     fi: &FrameInvariants<T>,
