@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2020, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -10,17 +10,20 @@
 #![allow(non_camel_case_types)]
 #![allow(dead_code)]
 
-use crate::serialize::{Deserialize, Serialize};
-
 use self::BlockSize::*;
 use self::TxSize::*;
 use crate::context::*;
 use crate::frame::*;
 use crate::predict::*;
 use crate::recon_intra::*;
+use crate::serialize::{Deserialize, Serialize};
 use crate::tiling::*;
 use crate::transform::TxSize;
 use crate::util::*;
+use thiserror::Error;
+
+use std::mem::transmute;
+use std::mem::MaybeUninit;
 
 // LAST_FRAME through ALTREF_FRAME correspond to slots 0-6.
 #[derive(PartialEq, Eq, PartialOrd, Copy, Clone, Debug)]
@@ -37,7 +40,11 @@ pub enum RefType {
 }
 
 impl RefType {
-  // convert to a ref list index, 0-6 (INTER_REFS_PER_FRAME)
+  /// convert to a ref list index, 0-6 (`INTER_REFS_PER_FRAME`)
+  ///
+  /// # Panics
+  ///
+  /// - If the ref type is a None or Intra frame
   #[inline]
   pub fn to_index(self) -> usize {
     match self {
@@ -62,6 +69,7 @@ impl RefType {
 
 use self::RefType::*;
 use std::fmt;
+use std::fmt::Display;
 
 pub const ALL_INTER_REFS: [RefType; 7] = [
   LAST_FRAME,
@@ -102,7 +110,7 @@ pub const REF_FRAMES: usize = 1 << REF_FRAMES_LOG2;
 pub const REF_CONTEXTS: usize = 3;
 pub const MVREF_ROW_COLS: usize = 3;
 
-#[derive(Copy, Clone, PartialEq, PartialOrd, Debug)]
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Debug)]
 pub enum PartitionType {
   PARTITION_NONE,
   PARTITION_HORZ,
@@ -141,7 +149,15 @@ pub enum BlockSize {
   BLOCK_32X8,
   BLOCK_16X64,
   BLOCK_64X16,
-  BLOCK_INVALID,
+}
+
+#[derive(Debug, Error, Copy, Clone, Eq, PartialEq)]
+pub struct InvalidBlockSize;
+
+impl Display for InvalidBlockSize {
+  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+    f.write_str("invalid block size")
+  }
 }
 
 impl PartialOrd for BlockSize {
@@ -160,40 +176,55 @@ impl PartialOrd for BlockSize {
   }
 }
 
+#[cfg(test)]
+impl Default for BlockSize {
+  fn default() -> Self {
+    BlockSize::BLOCK_64X64
+  }
+}
+
 impl BlockSize {
   pub const BLOCK_SIZES_ALL: usize = 22;
   pub const BLOCK_SIZES: usize = BlockSize::BLOCK_SIZES_ALL - 6; // BLOCK_SIZES_ALL minus 4:1 non-squares, six of them
 
   #[inline]
-  pub fn from_width_and_height_opt(w: usize, h: usize) -> Option<BlockSize> {
+  /// # Errors
+  ///
+  /// - Returns `InvalidBlockSize` if the given `w` and `h` do not produce
+  ///   a valid block size.
+  pub fn from_width_and_height_opt(
+    w: usize, h: usize,
+  ) -> Result<BlockSize, InvalidBlockSize> {
     match (w, h) {
-      (4, 4) => Some(BLOCK_4X4),
-      (4, 8) => Some(BLOCK_4X8),
-      (8, 4) => Some(BLOCK_8X4),
-      (8, 8) => Some(BLOCK_8X8),
-      (8, 16) => Some(BLOCK_8X16),
-      (16, 8) => Some(BLOCK_16X8),
-      (16, 16) => Some(BLOCK_16X16),
-      (16, 32) => Some(BLOCK_16X32),
-      (32, 16) => Some(BLOCK_32X16),
-      (32, 32) => Some(BLOCK_32X32),
-      (32, 64) => Some(BLOCK_32X64),
-      (64, 32) => Some(BLOCK_64X32),
-      (64, 64) => Some(BLOCK_64X64),
-      (64, 128) => Some(BLOCK_64X128),
-      (128, 64) => Some(BLOCK_128X64),
-      (128, 128) => Some(BLOCK_128X128),
-      (4, 16) => Some(BLOCK_4X16),
-      (16, 4) => Some(BLOCK_16X4),
-      (8, 32) => Some(BLOCK_8X32),
-      (32, 8) => Some(BLOCK_32X8),
-      (16, 64) => Some(BLOCK_16X64),
-      (64, 16) => Some(BLOCK_64X16),
-      _ => None,
+      (4, 4) => Ok(BLOCK_4X4),
+      (4, 8) => Ok(BLOCK_4X8),
+      (4, 16) => Ok(BLOCK_4X16),
+      (8, 4) => Ok(BLOCK_8X4),
+      (8, 8) => Ok(BLOCK_8X8),
+      (8, 16) => Ok(BLOCK_8X16),
+      (8, 32) => Ok(BLOCK_8X32),
+      (16, 4) => Ok(BLOCK_16X4),
+      (16, 8) => Ok(BLOCK_16X8),
+      (16, 16) => Ok(BLOCK_16X16),
+      (16, 32) => Ok(BLOCK_16X32),
+      (16, 64) => Ok(BLOCK_16X64),
+      (32, 8) => Ok(BLOCK_32X8),
+      (32, 16) => Ok(BLOCK_32X16),
+      (32, 32) => Ok(BLOCK_32X32),
+      (32, 64) => Ok(BLOCK_32X64),
+      (64, 16) => Ok(BLOCK_64X16),
+      (64, 32) => Ok(BLOCK_64X32),
+      (64, 64) => Ok(BLOCK_64X64),
+      (64, 128) => Ok(BLOCK_64X128),
+      (128, 64) => Ok(BLOCK_128X64),
+      (128, 128) => Ok(BLOCK_128X128),
+      _ => Err(InvalidBlockSize),
     }
   }
 
-  #[inline]
+  /// # Panics
+  ///
+  /// - If the given `w` and `h` do not produce a valid block size.
   pub fn from_width_and_height(w: usize, h: usize) -> BlockSize {
     Self::from_width_and_height_opt(w, h).unwrap()
   }
@@ -205,12 +236,18 @@ impl BlockSize {
   }
 
   #[inline]
-  pub fn width(self) -> usize {
+  pub const fn width(self) -> usize {
     1 << self.width_log2()
   }
 
+  /// width * height
   #[inline]
-  pub fn width_log2(self) -> usize {
+  pub const fn area(self) -> usize {
+    self.width() * self.height()
+  }
+
+  #[inline]
+  pub const fn width_log2(self) -> usize {
     match self {
       BLOCK_4X4 | BLOCK_4X8 | BLOCK_4X16 => 2,
       BLOCK_8X4 | BLOCK_8X8 | BLOCK_8X16 | BLOCK_8X32 => 3,
@@ -218,17 +255,16 @@ impl BlockSize {
       BLOCK_32X8 | BLOCK_32X16 | BLOCK_32X32 | BLOCK_32X64 => 5,
       BLOCK_64X16 | BLOCK_64X32 | BLOCK_64X64 | BLOCK_64X128 => 6,
       BLOCK_128X64 | BLOCK_128X128 => 7,
-      BLOCK_INVALID => unreachable!(),
     }
   }
 
   #[inline]
-  pub fn width_mi_log2(self) -> usize {
+  pub const fn width_mi_log2(self) -> usize {
     self.width_log2() - 2
   }
 
   #[inline]
-  pub fn width_mi(self) -> usize {
+  pub const fn width_mi(self) -> usize {
     self.width() >> MI_SIZE_LOG2
   }
 
@@ -239,12 +275,12 @@ impl BlockSize {
   }
 
   #[inline]
-  pub fn height(self) -> usize {
+  pub const fn height(self) -> usize {
     1 << self.height_log2()
   }
 
   #[inline]
-  pub fn height_log2(self) -> usize {
+  pub const fn height_log2(self) -> usize {
     match self {
       BLOCK_4X4 | BLOCK_8X4 | BLOCK_16X4 => 2,
       BLOCK_4X8 | BLOCK_8X8 | BLOCK_16X8 | BLOCK_32X8 => 3,
@@ -252,17 +288,16 @@ impl BlockSize {
       BLOCK_8X32 | BLOCK_16X32 | BLOCK_32X32 | BLOCK_64X32 => 5,
       BLOCK_16X64 | BLOCK_32X64 | BLOCK_64X64 | BLOCK_128X64 => 6,
       BLOCK_64X128 | BLOCK_128X128 => 7,
-      BLOCK_INVALID => unreachable!(),
     }
   }
 
   #[inline]
-  pub fn height_mi_log2(self) -> usize {
+  pub const fn height_mi_log2(self) -> usize {
     self.height_log2() - 2
   }
 
   #[inline]
-  pub fn height_mi(self) -> usize {
+  pub const fn height_mi(self) -> usize {
     self.height() >> MI_SIZE_LOG2
   }
 
@@ -273,7 +308,7 @@ impl BlockSize {
   }
 
   #[inline]
-  pub fn tx_size(self) -> TxSize {
+  pub const fn tx_size(self) -> TxSize {
     match self {
       BLOCK_4X4 => TX_4X4,
       BLOCK_4X8 => TX_4X8,
@@ -293,15 +328,21 @@ impl BlockSize {
       BLOCK_32X8 => TX_32X8,
       BLOCK_16X64 => TX_16X64,
       BLOCK_64X16 => TX_64X16,
-      BLOCK_INVALID => unreachable!(),
       _ => TX_64X64,
     }
   }
 
-  /// Source: Subsampled_Size (AV1 specification section 5.11.38)
+  /// Source: `Subsampled_Size` (AV1 specification section 5.11.38)
+  ///
+  /// # Errors
+  ///
+  /// - Returns `InvalidBlockSize` if the given block size cannot
+  ///   be subsampled in the requested way.
   #[inline]
-  pub fn subsampled_size(self, xdec: usize, ydec: usize) -> BlockSize {
-    match (xdec, ydec) {
+  pub const fn subsampled_size(
+    self, xdec: usize, ydec: usize,
+  ) -> Result<BlockSize, InvalidBlockSize> {
+    Ok(match (xdec, ydec) {
       (0, 0) /* 4:4:4 */ => self,
       (1, 0) /* 4:2:2 */ => match self {
         BLOCK_4X4 | BLOCK_8X4 => BLOCK_4X4,
@@ -317,7 +358,7 @@ impl BlockSize {
         BLOCK_64X64 => BLOCK_32X64,
         BLOCK_128X64 => BLOCK_64X64,
         BLOCK_128X128 => BLOCK_64X128,
-        _ => BLOCK_INVALID
+        _ => return Err(InvalidBlockSize),
       },
       (1, 1) /* 4:2:0 */ => match self {
         BLOCK_4X4 | BLOCK_4X8 | BLOCK_8X4 | BLOCK_8X8 => BLOCK_4X4,
@@ -337,18 +378,19 @@ impl BlockSize {
         BLOCK_64X128 => BLOCK_32X64,
         BLOCK_128X64 => BLOCK_64X32,
         BLOCK_128X128 => BLOCK_64X64,
-        _ => BLOCK_INVALID
       },
-      _ => unreachable!()
-    }
+      _ => return Err(InvalidBlockSize),
+    })
   }
 
+  /// # Panics
+  ///
+  /// Will panic if the subsampling is not possible
   #[inline]
   pub fn largest_chroma_tx_size(self, xdec: usize, ydec: usize) -> TxSize {
-    let plane_bsize = self.subsampled_size(xdec, ydec);
-    if plane_bsize == BLOCK_INVALID {
-      panic!("invalid block size for this subsampling mode");
-    }
+    let plane_bsize = self
+      .subsampled_size(xdec, ydec)
+      .expect("invalid block size for this subsampling mode");
 
     let chroma_tx_size = max_txsize_rect_lookup[plane_bsize as usize];
 
@@ -356,27 +398,35 @@ impl BlockSize {
   }
 
   #[inline]
-  pub fn is_sqr(self) -> bool {
+  pub const fn is_sqr(self) -> bool {
     self.width_log2() == self.height_log2()
   }
 
   #[inline]
-  pub fn is_sub8x8(self, xdec: usize, ydec: usize) -> bool {
+  pub const fn is_sub8x8(self, xdec: usize, ydec: usize) -> bool {
     xdec != 0 && self.width_log2() == 2 || ydec != 0 && self.height_log2() == 2
   }
 
   #[inline]
-  pub fn sub8x8_offset(self, xdec: usize, ydec: usize) -> (isize, isize) {
+  pub const fn sub8x8_offset(
+    self, xdec: usize, ydec: usize,
+  ) -> (isize, isize) {
     let offset_x = if xdec != 0 && self.width_log2() == 2 { -1 } else { 0 };
     let offset_y = if ydec != 0 && self.height_log2() == 2 { -1 } else { 0 };
 
     (offset_x, offset_y)
   }
 
-  pub fn subsize(self, partition: PartitionType) -> BlockSize {
+  /// # Errors
+  ///
+  /// - Returns `InvalidBlockSize` if the block size cannot be split
+  ///   in the requested way.
+  pub const fn subsize(
+    self, partition: PartitionType,
+  ) -> Result<BlockSize, InvalidBlockSize> {
     use PartitionType::*;
 
-    match partition {
+    Ok(match partition {
       PARTITION_NONE => self,
       PARTITION_SPLIT => match self {
         BLOCK_8X8 => BLOCK_4X4,
@@ -384,7 +434,7 @@ impl BlockSize {
         BLOCK_32X32 => BLOCK_16X16,
         BLOCK_64X64 => BLOCK_32X32,
         BLOCK_128X128 => BLOCK_64X64,
-        _ => BLOCK_INVALID,
+        _ => return Err(InvalidBlockSize),
       },
       PARTITION_HORZ | PARTITION_HORZ_A | PARTITION_HORZ_B => match self {
         BLOCK_8X8 => BLOCK_8X4,
@@ -392,7 +442,7 @@ impl BlockSize {
         BLOCK_32X32 => BLOCK_32X16,
         BLOCK_64X64 => BLOCK_64X32,
         BLOCK_128X128 => BLOCK_128X64,
-        _ => BLOCK_INVALID,
+        _ => return Err(InvalidBlockSize),
       },
       PARTITION_VERT | PARTITION_VERT_A | PARTITION_VERT_B => match self {
         BLOCK_8X8 => BLOCK_4X8,
@@ -400,31 +450,36 @@ impl BlockSize {
         BLOCK_32X32 => BLOCK_16X32,
         BLOCK_64X64 => BLOCK_32X64,
         BLOCK_128X128 => BLOCK_64X128,
-        _ => BLOCK_INVALID,
+        _ => return Err(InvalidBlockSize),
       },
       PARTITION_HORZ_4 => match self {
         BLOCK_16X16 => BLOCK_16X4,
         BLOCK_32X32 => BLOCK_32X8,
         BLOCK_64X64 => BLOCK_64X16,
-        _ => BLOCK_INVALID,
+        _ => return Err(InvalidBlockSize),
       },
       PARTITION_VERT_4 => match self {
         BLOCK_16X16 => BLOCK_4X16,
         BLOCK_32X32 => BLOCK_8X32,
         BLOCK_64X64 => BLOCK_16X64,
-        _ => BLOCK_INVALID,
+        _ => return Err(InvalidBlockSize),
       },
-      _ => BLOCK_INVALID,
-    }
+      _ => return Err(InvalidBlockSize),
+    })
   }
 
-  pub fn is_rect_tx_allowed(self) -> bool {
-    match self {
-      BLOCK_4X4 | BLOCK_8X8 | BLOCK_16X16 | BLOCK_32X32 | BLOCK_64X64
-      | BLOCK_64X128 | BLOCK_128X64 | BLOCK_128X128 => false,
-      BLOCK_INVALID => unreachable!(),
-      _ => true,
-    }
+  pub const fn is_rect_tx_allowed(self) -> bool {
+    !matches!(
+      self,
+      BLOCK_4X4
+        | BLOCK_8X8
+        | BLOCK_16X16
+        | BLOCK_32X32
+        | BLOCK_64X64
+        | BLOCK_64X128
+        | BLOCK_128X64
+        | BLOCK_128X128
+    )
   }
 }
 
@@ -456,7 +511,6 @@ impl fmt::Display for BlockSize {
         BlockSize::BLOCK_32X8 => "32x8",
         BlockSize::BLOCK_16X64 => "16x64",
         BlockSize::BLOCK_64X16 => "64x16",
-        BlockSize::BLOCK_INVALID => "Invalid",
       }
     )
   }
@@ -481,7 +535,7 @@ pub static RAV1E_PARTITION_TYPES: &[PartitionType] = &[
   PartitionType::PARTITION_SPLIT,
 ];
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum GlobalMVMode {
   IDENTITY = 0,    // identity transformation, 0-parameter
   TRANSLATION = 1, // translational motion 2-parameter
@@ -489,7 +543,7 @@ pub enum GlobalMVMode {
   AFFINE = 3,      // affine, 6-parameter
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum MvSubpelPrecision {
   MV_SUBPEL_NONE = -1,
   MV_SUBPEL_LOW_PRECISION = 0,
@@ -499,7 +553,7 @@ pub enum MvSubpelPrecision {
 /* Symbols for coding which components are zero jointly */
 pub const MV_JOINTS: usize = 4;
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum MvJointType {
   MV_JOINT_ZERO = 0,   /* Zero vector */
   MV_JOINT_HNZVZ = 1,  /* Vert zero, hor nonzero */
@@ -548,7 +602,47 @@ fn supersample_chroma_bsize(
   }
 }
 
-pub fn get_intra_edges<T: Pixel>(
+type IntraEdgeBuffer<T> = Aligned<[MaybeUninit<T>; 4 * MAX_TX_SIZE + 1]>;
+
+#[cfg(any(test, feature = "bench"))]
+type IntraEdgeMock<T> = Aligned<[T; 4 * MAX_TX_SIZE + 1]>;
+
+pub struct IntraEdge<'a, T: Pixel>(&'a [T], &'a [T], &'a [T]);
+
+impl<'a, T: Pixel> IntraEdge<'a, T> {
+  fn new(
+    edge_buf: &'a mut IntraEdgeBuffer<T>, init_left: usize, init_above: usize,
+  ) -> Self {
+    // SAFETY: Initialized in `get_intra_edges`.
+    let left = unsafe {
+      let begin_left = 2 * MAX_TX_SIZE - init_left;
+      let end_above = 2 * MAX_TX_SIZE + 1 + init_above;
+      slice_assume_init_mut(&mut edge_buf.data[begin_left..end_above])
+    };
+    let (left, top_left) = left.split_at(init_left);
+    let (top_left, above) = top_left.split_at(1);
+    Self(left, top_left, above)
+  }
+
+  pub const fn as_slices(&self) -> (&'a [T], &'a [T], &'a [T]) {
+    (self.0, self.1, self.2)
+  }
+
+  pub const fn top_left_ptr(&self) -> *const T {
+    self.1.as_ptr()
+  }
+
+  #[cfg(any(test, feature = "bench"))]
+  pub fn mock(edge_buf: &'a IntraEdgeMock<T>) -> Self {
+    let left = &edge_buf.data[..];
+    let (left, top_left) = left.split_at(2 * MAX_TX_SIZE);
+    let (top_left, above) = top_left.split_at(1);
+    Self(left, top_left, above)
+  }
+}
+
+pub fn get_intra_edges<'a, T: Pixel>(
+  edge_buf: &'a mut IntraEdgeBuffer<T>,
   dst: &PlaneRegion<'_, T>,
   partition_bo: TileBlockOffset, // partition bo, BlockOffset
   bx: usize,
@@ -560,12 +654,12 @@ pub fn get_intra_edges<T: Pixel>(
   opt_mode: Option<PredictionMode>,
   enable_intra_edge_filter: bool,
   intra_param: IntraParam,
-) -> Aligned<[T; 4 * MAX_TX_SIZE + 1]> {
+) -> IntraEdge<'a, T> {
+  let mut init_left: usize = 0;
+  let mut init_above: usize = 0;
+
   let plane_cfg = &dst.plane_cfg;
 
-  let mut edge_buf: Aligned<[T; 4 * MAX_TX_SIZE + 1]> =
-    Aligned::uninitialized();
-  //Aligned::new([T::cast_from(0); 4 * MAX_TX_SIZE + 1]);
   let base = 128u16 << (bit_depth - 8);
 
   {
@@ -628,20 +722,21 @@ pub fn get_intra_edges<T: Pixel>(
       if x != 0 {
         for i in 0..txh {
           debug_assert!(y + i < rect_h);
-          left[2 * MAX_TX_SIZE - 1 - i] = dst[y + i][x - 1];
+          left[2 * MAX_TX_SIZE - 1 - i].write(dst[y + i][x - 1]);
         }
         if txh < tx_size.height() {
           let val = dst[y + txh - 1][x - 1];
           for i in txh..tx_size.height() {
-            left[2 * MAX_TX_SIZE - 1 - i] = val;
+            left[2 * MAX_TX_SIZE - 1 - i].write(val);
           }
         }
       } else {
         let val = if y != 0 { dst[y - 1][0] } else { T::cast_from(base + 1) };
         for v in left[2 * MAX_TX_SIZE - tx_size.height()..].iter_mut() {
-          *v = val
+          v.write(val);
         }
       }
+      init_left += tx_size.height();
     }
 
     // Needs top
@@ -652,38 +747,25 @@ pub fn get_intra_edges<T: Pixel>(
         tx_size.width()
       };
       if y != 0 {
-        above[..txw].copy_from_slice(&dst[y - 1][x..x + txw]);
+        above[..txw].copy_from_slice(
+          // SAFETY: &[T] and &[MaybeUninit<T>] have the same layout
+          unsafe {
+            transmute::<&[T], &[MaybeUninit<T>]>(&dst[y - 1][x..x + txw])
+          },
+        );
         if txw < tx_size.width() {
           let val = dst[y - 1][x + txw - 1];
           for i in txw..tx_size.width() {
-            above[i] = val;
+            above[i].write(val);
           }
         }
       } else {
         let val = if x != 0 { dst[0][x - 1] } else { T::cast_from(base - 1) };
         for v in above[..tx_size.width()].iter_mut() {
-          *v = val;
+          v.write(val);
         }
       }
-    }
-
-    // Needs top-left
-    if needs_topleft {
-      top_left[0] = match (x, y) {
-        (0, 0) => T::cast_from(base),
-        (_, 0) => dst[0][x - 1],
-        (0, _) => dst[y - 1][0],
-        _ => dst[y - 1][x - 1],
-      };
-
-      let (w, h) = (tx_size.width(), tx_size.height());
-      if needs_topleft_filter && w + h >= 24 {
-        let (l, a, tl): (u32, u32, u32) =
-          (left[left.len() - 1].into(), above[0].into(), top_left[0].into());
-        let s = l * 5 + tl * 6 + a * 5;
-
-        top_left[0] = T::cast_from((s + (1 << 3)) >> 4);
-      }
+      init_above += tx_size.width();
     }
 
     let bx4 = bx * (tx_size.width() >> MI_SIZE_LOG2); // bx,by are in tx block indices
@@ -729,8 +811,13 @@ pub fn get_intra_edges<T: Pixel>(
         0
       };
       if num_avail > 0 {
-        above[tx_size.width()..tx_size.width() + num_avail].copy_from_slice(
-          &dst[y - 1][x + tx_size.width()..x + tx_size.width() + num_avail],
+        above[tx_size.width()..][..num_avail].copy_from_slice(
+          // SAFETY: &[T] and &[MaybeUninit<T>] have the same layout
+          unsafe {
+            transmute::<&[T], &[MaybeUninit<T>]>(
+              &dst[y - 1][x + tx_size.width()..][..num_avail],
+            )
+          },
         );
       }
       if num_avail < tx_size.height() {
@@ -742,8 +829,12 @@ pub fn get_intra_edges<T: Pixel>(
           *v = val;
         }
       }
+      init_above += tx_size.height();
     }
 
+    // SAFETY: The blocks above have initialized the first `init_above` items.
+    let above = unsafe { slice_assume_init_mut(&mut above[..init_above]) };
+
     // Needs bottom left
     if needs_bottomleft {
       debug_assert!(plane_cfg.xdec <= 1 && plane_cfg.ydec <= 1);
@@ -766,8 +857,8 @@ pub fn get_intra_edges<T: Pixel>(
       };
       if num_avail > 0 {
         for i in 0..num_avail {
-          left[2 * MAX_TX_SIZE - tx_size.height() - 1 - i] =
-            dst[y + tx_size.height() + i][x - 1];
+          left[2 * MAX_TX_SIZE - tx_size.height() - 1 - i]
+            .write(dst[y + tx_size.height() + i][x - 1]);
         }
       }
       if num_avail < tx_size.width() {
@@ -779,9 +870,36 @@ pub fn get_intra_edges<T: Pixel>(
           *v = val;
         }
       }
+      init_left += tx_size.width();
+    }
+
+    // SAFETY: The blocks above have initialized last `init_left` items.
+    let left = unsafe {
+      slice_assume_init_mut(&mut left[2 * MAX_TX_SIZE - init_left..])
+    };
+
+    // Needs top-left
+    if needs_topleft {
+      let top_left = top_left[0].write(match (x, y) {
+        (0, 0) => T::cast_from(base),
+        (_, 0) => dst[0][x - 1],
+        (0, _) => dst[y - 1][0],
+        _ => dst[y - 1][x - 1],
+      });
+
+      let (w, h) = (tx_size.width(), tx_size.height());
+      if needs_topleft_filter && w + h >= 24 {
+        let (l, a, tl): (u32, u32, u32) =
+          (left[left.len() - 1].into(), above[0].into(), (*top_left).into());
+        let s = l * 5 + tl * 6 + a * 5;
+
+        *top_left = T::cast_from((s + (1 << 3)) >> 4);
+      }
+    } else {
+      top_left[0].write(T::cast_from(base));
     }
   }
-  edge_buf
+  IntraEdge::new(edge_buf, init_left, init_above)
 }
 
 pub fn has_tr(bo: TileBlockOffset, bsize: BlockSize) -> bool {
@@ -898,3 +1016,51 @@ pub fn has_bl(bo: TileBlockOffset, bsize: BlockSize) -> bool {
 
   has_bl
 }
+
+#[cfg(test)]
+mod tests {
+  use crate::partition::BlockSize::*;
+  use crate::partition::{BlockSize, InvalidBlockSize};
+
+  #[test]
+  fn from_wh_matches_naive() {
+    fn from_wh_opt_naive(
+      w: usize, h: usize,
+    ) -> Result<BlockSize, InvalidBlockSize> {
+      match (w, h) {
+        (4, 4) => Ok(BLOCK_4X4),
+        (4, 8) => Ok(BLOCK_4X8),
+        (8, 4) => Ok(BLOCK_8X4),
+        (8, 8) => Ok(BLOCK_8X8),
+        (8, 16) => Ok(BLOCK_8X16),
+        (16, 8) => Ok(BLOCK_16X8),
+        (16, 16) => Ok(BLOCK_16X16),
+        (16, 32) => Ok(BLOCK_16X32),
+        (32, 16) => Ok(BLOCK_32X16),
+        (32, 32) => Ok(BLOCK_32X32),
+        (32, 64) => Ok(BLOCK_32X64),
+        (64, 32) => Ok(BLOCK_64X32),
+        (64, 64) => Ok(BLOCK_64X64),
+        (64, 128) => Ok(BLOCK_64X128),
+        (128, 64) => Ok(BLOCK_128X64),
+        (128, 128) => Ok(BLOCK_128X128),
+        (4, 16) => Ok(BLOCK_4X16),
+        (16, 4) => Ok(BLOCK_16X4),
+        (8, 32) => Ok(BLOCK_8X32),
+        (32, 8) => Ok(BLOCK_32X8),
+        (16, 64) => Ok(BLOCK_16X64),
+        (64, 16) => Ok(BLOCK_64X16),
+        _ => Err(InvalidBlockSize),
+      }
+    }
+
+    for w in 0..256 {
+      for h in 0..256 {
+        let a = BlockSize::from_width_and_height_opt(w, h);
+        let b = from_wh_opt_naive(w, h);
+
+        assert_eq!(a, b);
+      }
+    }
+  }
+}
